package net.sf.corn.gate;

import java.lang.reflect.Method;

/**
 * Execution advice wraps executions of target service methods. Advice
 * implementations are capable of modifying both incoming requests and outgoing
 * responses. {@link ServiceFilter} defines service and operation targets, so
 * you can add your advice into a filter and register it by using
 * {@link Registry#registerFilter} method on runtime. The other way of
 * definition and registration is putting it to a configuration file. The third
 * way of doing that is annotating an advice implementation with
 * {@link net.sf.corn.gate.GateServiceTarget} annotation, than you can either register that 
 * implementation by using {@link Registry}
 * 
 * @author Serhat Dirik
 */
public interface IExecutionAdvice {

	/**
	 *  This method is called just before the service operation is executed.
	 * @param request  Request wrapper
	 * @param serviceDef  Service Definition
	 * @param opDef  Operation Definition
	 * @param serviceInstance An actual instance of the service
	 * @param method  A Java method (operation) to be executed
	 */
	public void executeBeforeMethod(ServiceRequest request,
			ServiceDefinition serviceDef, OperationDefinition opDef,
			Object serviceInstance, Method method);

 
	/**
	 * This method is called just after the target service operation is
	 * executed. At this point you have a chance to manipulate the real
	 * operation result.
	 * @param request request wrapper
	 * @param response  The result of the method execution
	 * @param serviceDef  Service Definition
	 * @param opDef Operation Definition
	 * @param serviceInstance An actual instance of the service
	 * @param method A Java method (operation) that executed
	 * @return modified response
	 */
	public ServiceResponse executeAfterMethod(ServiceRequest request,
			ServiceResponse response, ServiceDefinition serviceDef,
			OperationDefinition opDef, Object serviceInstance, Method method);

}
