package net.sf.corn.gate.web;

import java.util.Enumeration;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import net.sf.corn.exception.ExceptionBuilder;
import net.sf.corn.gate.GateException;
import net.sf.corn.gate.IClientContext;
import net.sf.corn.gate.IDataStage;
import net.sf.corn.gate.IDataStage.Scope;
import net.sf.corn.gate.SimpleConcurrentDataStage;
/**
 * ClientContext implementation for web applications. If you're not a protocol developer for gate, 
 * never mind about this class. {@link net.sf.corn.gate.CallDelegator#getActiveClientContext} returns 
 * instance of this class when the client accessed to a service through an http based procol.
 * This implementation wraps the HttpServletRequest and binds the data stages as below:
 * <p>
 * <li>REQUEST: This data stage is bounded to Http Request</li>
 * <li>SESSION: This data stage is bounded to Http Session</li>
 * <li>APPLICATION: This data stage is bounded to Servlet Context</li>
 * <li>PLATFORM: This data stage is bounded to a static data store which can be accesible by anyone in the same JVM scope</li>
 * </p>
 * @author Serhat Dirik
 */
public class WebClientContext implements IClientContext {
 
	private static final long serialVersionUID = 7913390796772043119L;
	public static final String PROTOCOL_NAME = "http";
	private static IDataStage platformScope =  null;
	private IDataStage applicationScope =  null;
	private IDataStage requestScope = null;
	private IDataStage sessionScope = null;
	private HttpServletRequest httpRequest;
	private Map<String,Object> header = new ConcurrentHashMap<String, Object>();
	private Map<String,Object> requestHeader = new ConcurrentHashMap<String, Object>();
	
	static{
		platformScope = new SimpleConcurrentDataStage(IDataStage.Scope.PLATFORM);
	}
	
	public WebClientContext(HttpServletRequest httpRequest) {
		if(httpRequest == null){
			ExceptionBuilder.with(IllegalArgumentException.class).message("HttpRequest is null!").throwIt();
		}
		 this.httpRequest = httpRequest;
		 HttpSession session = httpRequest.getSession(true);
		 this.applicationScope = new WebAppDataStage(session.getServletContext());
		 this.sessionScope = new WebSessionDataStage(session);
		 this.requestScope  = new WebRequestDataStage(httpRequest);
	}
	
	@Override
	public String getClientUser() {
		return httpRequest.getRemoteUser();
	}

	@Override
	public boolean isUserInRole(String role) {
		return httpRequest.isUserInRole(role);
	}

	@Override
	public IDataStage getDataStage(Scope type) {
		if(type == null) {
			ExceptionBuilder.with(IllegalArgumentException.class).message("Null scope type is requested!").throwIt();
		}
		if(type.equals(Scope.PLATFORM)){
			return platformScope;
		}else if(type.equals(Scope.APPLICATION)){
			return applicationScope;
		}else if(type.equals(Scope.REQUEST)){
			return requestScope;
		}else if(type.equals(Scope.SESSION)){
			return sessionScope;
		}  
		return null;
	}

	@Override
	public String getProtocolName() {
		return PROTOCOL_NAME;
	}

	@Override
	public String getClientNetworkAddr() {
		return httpRequest.getRemoteAddr();
	}

	@Override
	public int getClientCommunicationPort() {
		return httpRequest.getRemotePort();
	}

	@Override
	public String getClientHostName() {
		return httpRequest.getRemoteHost();
	}

	@Override
	public boolean isLocal() {
		return false;
	}

	@Override
	public boolean isProtocolSecure() {
		return httpRequest.isSecure();
	}

	@Override
	public String getServerName() {
		return httpRequest.getServerName();
	}


	@Override
	public int getServerPort() {
		return httpRequest.getServerPort();
	}


	@Override
	public String getServerNetworkAddr() {
		return httpRequest.getLocalAddr();
	}
 
	
	@Override
	public Object addToResponseHeader(String key, Object value) {
		return header.put(key, value);
	}
	@Override
	public Object removeFromResponseHeader(String key) {
		return header.remove(key);
	}
	@Override
	public Set<String> responseHeaderKeySet() {
		return header.keySet();
	}
	@Override
	public Object getFromResponseHeader(String key) {
		return header.get(key);
	}
	@Override
	public boolean containsResponseHeaderKey(String key) {
		return header.containsKey(key);
	}

	
	public Object addToRequestHeader(String key, Object value) {
		return requestHeader.put(key, value);
	}
	
	public Object removeFromRequestHeader(String key) {
		return requestHeader.remove(key);
	}
	public boolean containsRequestHeaderKey(String key) {
		return requestHeader.containsKey(key);
	}
	@Override
	public Object getFromRequestHeader(String key) {
		 Object rVal = requestHeader.get(key);
		 return rVal != null ? rVal : httpRequest.getHeader(key);
	}

	@Override
	public Set<String> requestHeaderKeySet() {
	   Set<String> keySet = new HashSet<>();
	   keySet.addAll(requestHeader.keySet());
	   Enumeration<String> names =  httpRequest.getHeaderNames();
	   while(names.hasMoreElements()){
		   keySet.add(names.nextElement());
	   }
	   return keySet;
	}
	
	@Override
	public String toString() {
		
		return new StringBuilder().append("Context:").append(" user:").append(getClientUser())
				.append(", client address:").append(getClientNetworkAddr()).
				append(", header:").append(requestHeader.toString()).toString();
	}

	@Override
	public void login(String username, String password) throws GateException {
		try {
			this.httpRequest.login(username, password);
		} catch (Exception e) {
			throw ExceptionBuilder.with(GateException.class,e).message(e.getMessage()).build().code(GateException.CODE_SECURITY_LOGIN_FAILED);
		}
		
	}

	@Override
	public void logout() throws GateException {
		try {
			this.httpRequest.logout();
		} catch (Exception e) {
			throw ExceptionBuilder.with(GateException.class,e).message(e.getMessage()).build().code(GateException.CODE_SECURITY_LOGOUT_FAILED);
		}
		
	}

}
